import cv2
import numpy as np
from ultralytics import YOLO

# coordinates 저장하는 리스트
coordinates = []

# 웹캠 초기화
cap = cv2.VideoCapture(2)
model = YOLO('best(2).pt')

# Define a callback function to capture mouse click coordinates
def get_coordinates(event, x, y, flags, param):
    if event == cv2.EVENT_LBUTTONDOWN:  # Left mouse button click
        if len(coordinates) < 4:  # Only collect four clicks
            coordinates.append((x, y))
            print(f"Coordinate {len(coordinates)}: ({x}, {y})")
        # Check if we've collected four clicks
        if len(coordinates) == 4:
            # Stop further clicks by deactivating the callback
            cv2.setMouseCallback('Camera', lambda *args: None)  # Disables further clicks
            
cv2.namedWindow('Camera')
cv2.setMouseCallback('Camera', get_coordinates)

while True:
    red_screen = False  # 화면 색상 변경을 위한 변수
    ret, frame = cap.read()
    if not ret:
        break
    results = model(frame)
    
    # 각 검출된 객체에 대해 바운딩 박스와 레이블 표시
    for result in results[0].boxes:
        # 바운딩 박스 좌표 및 예측 클래스와 확률
        x1, y1, x2, y2 = map(int, result.xyxy[0])  # 바운딩 박스 좌표
        confidence = result.conf[0]  # 예측 확률
        class_id = int(result.cls[0])  # 클래스 ID
        label = f"{model.names[class_id]} {confidence:.2f}"
        # 객체의 중심 좌표 계산
        center_x = (x1 + x2) // 2
        center_y = (y1 + y2) // 2
        # 다각형 경계와 충돌 확인
        if len(coordinates) == 4:
            pts = np.array(coordinates, np.int32).reshape((-1, 1, 2))
            if cv2.pointPolygonTest(pts, (center_x, center_y), False) > 0:
                red_screen = True  # 충돌이 발생하면 red_screen 활성화
        # 바운딩 박스와 레이블을 프레임에 그리기
        cv2.rectangle(frame, (x1, y1), (x2, y2), (0, 255, 0), 2)  # 초록색 바운딩 박스
        cv2.putText(frame, label, (x1, y1 - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 255, 0), 2)
    # Draw each collected coordinate on the frame
    for (x, y) in coordinates:
        cv2.circle(frame, (x, y), 5, (0, 0, 255), -1)  # Red circle with radius 5
    # 다각형 그리기
    if len(coordinates) == 4:
        cv2.polylines(frame, [pts], isClosed=True, color=(0, 255, 0), thickness=2)
    # 충돌이 발생한 경우 빨간색 오버레이 추가
    if red_screen:
        overlay = frame.copy()
        overlay[:] = (0, 0, 255)  # 빨간색
        alpha = 0.4  # 투명도 설정
        frame = cv2.addWeighted(overlay, alpha, frame, 1 - alpha, 0)
    cv2.imshow('Camera', frame)
    # Press 'q' to exit
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break
# 프로그램 종료 시 웹캠 해제
cap.release()
cv2.destroyAllWindows()